--[[

GOSSIPMONGER FILTER SYSTEM

For use with the Gossipmonger addon, by Tuhljin.

To add a filter or change its properties, call:

Gossipmonger.SetFilter(func[, name, desc, priority, enabled])

Arguments:
  func           Function. The filter function. See below for a description of what your function's signature
                 should be.
  name           String. The name of your filter. Defaults to "Unnamed Filter".
  desc           String. A description of your filter. Defaults to "".
  priority       Number or nil. Determines when this filter will be checked in relation to other filters, since
                 as soon as one filter gives something non-nil for both of its first two returns, these checks
                 stop. Filters with larger numbers go first. Defaults to 50. The default filter (which mimics
                 standard WoW behavior) is set at priority 0.
  enabled        Boolean. Whether this filter function should be called by Gossipmonger. Defaults to false.

If a filter using the given function already exists, a new one isn't added. Instead, its properties are changed
to match those given by the other arguments.

You can pass nil for any of the above arguments except for func. If the filter is new, then default values (as
indicated above) are used as appropriate. Otherwise, the property associated with that argument remains
unchanged, letting you change three or fewer properties instead of all four if desired.


The signature of your function should be:

type, option, followup, override = MyFilter(mobID, NPCName, text, numAvailable, available, numActive, active,
                                            numGossip, gossip, isFollowup)
 Arguments:
  mobID          Number. The NPC's ID.
  NPCName        String. The NPC's name.
  text           String. The gossip text given by the dialog.
  numAvailable   Number. The number of available quests.
  available      Table. A table giving the details of the available quests. See Options Table, below.
  numActive      Number. The number of active quests (those in the player's quest log that are turned in here).
  active         Table. A table giving the details of the active quests. See Options Table, below.
  numGossip      Number. The number of gossip (AKA conversation/dialog) options.
  gossip         Table. A table giving the details of the gossip options. See Options Table, below.
  isFollowup     True or nil. If true, the call to this function is a followup call. (See the followup return.)

 Returns:
  type           Number, nil or false. If nil or false, this filter doesn't specify a reaction. Return 1 to
                 specify an available quest, 2 for an active quest, 3 for a gossip option, or 0 to select nothing
                 (causing the default gossip frame to show).
  option         Number, nil or false. If nil or false, this filter doesn't specify a reaction. Otherwise, this
                 should be the index of the option (whose type is given by by the previous return) to select or
                 0, which indicates that nothing should be selected (causing the default gossip frame to show).
                 This return is ignored if the first return, type, is nil, false, or 0.
  followup       Function or nil. If a function, then if selecting the option given by the first two returns
                 results in more gossip options appearing, the given function will be called to handle making
                 another automatic selection in the same manner as a normal filter call is made.
  override       Boolean or nil. Whether the gossip bubble should be forced to appear (if this is true) or not
                 appear (if this is false). If nil, the player's normal options are used. If this is false,
                 the player will not be informed of a skipped greeting using the chat log.


Other Functions
---------------

For the following, the argument "filter" is as follows:
  filter         Function or string. The filter function or its name. Note that while a function can only be part
                 of a single filter, it's possible for a given name to be used by more than one, so the wrong
                 filter might be used in such a case.

func, name, desc, priority, enabled = Gossipmonger.GetFilterInfo(filter)
  Get information about the given filter.

 Returns:
  func           Function. The filter function.
  name           String. The name of the filter.
  desc           String. The filter's description.
  priority       Number. The filter's priority.
  enabled        Boolean. Whether this filter function should be called by Gossipmonger.

success = Gossipmonger.EnableFilter(filter)
  Enable the given filter. Returns true if the filter was found and enabled.

success = Gossipmonger.DisableFilter(filter)
  Disable the given filter. Returns true if the filter was found and disabled.


Options Table
-------------

The options tables let your filter know the details about the available options given by the NPC at this point.
There are two types of options tables: Quest options (used for both available and active quests), and gossip
options. Both types consist of key-value pairs, one of which is the key "list" which points to a table (an array)
which gives the raw returns from GetGossipAvailableQuests(), GetGossipActiveQuests() or GetGossipOptions(), as
appropriate. Iterating through this list can be a hassle, though, and it isn't efficient if many different filters
are going to be called, so Gossipmonger provides another way to get data about these options using keys in the
form of "<label><#>", where <label> indicates what this data refers to (e.g. "title"), and <#> is the option
number to which it belongs.

Quest options tables have the following keys for each available option:

  title<#>       String. The name of the quest.
  level<#>       Number. The level of the quest.
  isLowLevel<#>  Boolean. True if this quest is considered to be low level for this player.

Gossip options tables have the following keys for each available option:

  title<#>       String. The title of the gossip item.
  type<#>        String. The gossip type, as the even-numbered returns given by GetGossipOptions(), e.g.,
                 "banker", "vendor", or "gossip". See: http://www.wowwiki.com/API_GetGossipOptions

NOTE: You should probably consider the tables passed to your filter function as "read only": Do not manipulate
them or you may break the functionality of other filters. (Actually forcing them to be "read only" using a
metatable was decided against, so just be sure you know what you're doing.)

--]]



local L = GOSSIPMONGER_STRINGS


local shouldSkipText = Gossipmonger.FiltersData.ShouldSkipText
local shouldSkipNPC = Gossipmonger.FiltersData.ShouldSkipNPC


-- FILTERS ENABLED BY DEFAULT
-----------------------------------

Gossipmonger.StandardFilters = {}


-- THE PRIMARY DEFAULT FILTER ("WoW Default"):
local function FILTER_Default(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip)
-- This mimics the behavior of the WoW client without addons: If there aren't any quests (available or active) AND
-- the gossip isn't specially marked by the server as not to be skipped AND
-- there is one and only one dialog option and it isn't of type "gossip", that option is selected.
  if (numAvailable == 0 and numActive == 0 and numGossip == 1 and not ForceGossip() and gossip.type1 ~= "gossip") then
    return 3, 1  -- 3 for gossip, 1 for option 1
  end
end
Gossipmonger.SetFilter(FILTER_Default, "", nil, 0, true)
Gossipmonger.StandardFilters["Default"] = FILTER_Default


-- SECONDARY DEFAULT FILTER "Accuracy":
local tab_Accuracy = Gossipmonger.FiltersData.Accuracy
local function FILTER_Accuracy(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip)
  if (numAvailable == 0 and numActive == 0 and numGossip == 1 and not ForceGossip()) then
    if (shouldSkipText(tab_Accuracy, text, gossip.type1, gossip.title1) or
		shouldSkipNPC(tab_Accuracy, mobID, gossip.type1)) then
      return 3, 1, nil, false  -- 3 for gossip, 1 for option 1, nil for no followup, false to prevent Gossip Bubble from showing
	end
  end
end
Gossipmonger.SetFilter(FILTER_Accuracy, "", nil, 1, true)
Gossipmonger.StandardFilters["Accuracy"] = FILTER_Accuracy


-- STANDARD BUT NON-DEFAULT FILTER "Mundane":
local tab_Mundane = Gossipmonger.FiltersData.Mundane
local function FILTER_Mundane(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip)
  if (numAvailable == 0 and numActive == 0 and numGossip == 1 and not ForceGossip()) then
    if (shouldSkipText(tab_Mundane, text, gossip.type1, gossip.title1) or
		shouldSkipNPC(tab_Mundane, mobID, gossip.type1)) then
	  return 3, 1, nil, false  -- 3 for gossip, 1 for option 1, nil for no followup, false to prevent Gossip Bubble from showing
	end
  end
end
Gossipmonger.SetFilter(FILTER_Mundane, "", nil, 1, true)
Gossipmonger.StandardFilters["Mundane"] = FILTER_Mundane


-- Consider a "REDUNDANT" filter? For things more useless than simply being repetitive (mundane), as it states something that will already be stated/shown explicitly.

-- Don't need to gather too many, though; when the Ignore feature is in, people will just set up ignore lists to handle a lot of stuff.
-- The standard filters are for the most common and bothersome issues so Gossipmonger works decently out of the box.


--[[
-- DON'T SHOW GOSSIP BUBBLE FOR BATTLEGROUND PORTALS:
local function IgnoreBGPortal(NPCName, text, numAvailable, available, numActive, active, numGossip, gossip)
  if (numAvailable == 0 and numActive == 0 and numGossip == 1 and gossip.type1 == "battlemaster" and
      NPCName:find(L.FILTER_NOPORTALCHAT_NAMEMATCH)) then
  -- Note that my filters are cautious. We don't want to auto-select something in the event that multiple options
  -- exist, even though at this time we know there is only one option.
    return 3, 1, nil, false
  end
end
Gossipmonger.SetFilter(IgnoreBGPortal, L.FILTER_NOPORTALCHAT, L.FILTER_NOPORTALCHAT_DESC, 1, true)
Gossipmonger.StandardFilters["BGPortal"] = FILTER_Default
--]]

--Gossipmonger.StandardFilters = { FILTER_Default, IgnoreBGPortal }
--Gossipmonger.StandardFilters = { FILTER_Default }



-- EXAMPLE FILTERS
------------------------

-- To use one of these filters, remove the --[[ and --]] from the lines that enclose it.


-- DON'T SHOW GOSSIP BUBBLE FOR STANDARD FLIGHT MASTER DIALOG:
--[[
local function IgnoreFlyWhere(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip)
  if (numAvailable == 0 and numActive == 0 and numGossip == 1 and gossip.type1 == "taxi" and
      text == "Where would you like to fly to?") then
    return 3, 1, nil, false
  end
end
Gossipmonger.SetFilter(IgnoreFlyWhere, "Ignore default flight master",
  "Don't show the gossip bubble with the standard flight master dialog, \"Where would you like to fly to?\"", 1, true)
--]]


-- HOLD ALT WHEN TALKING TO GUARDS TO FIND YOUR CLASS TRAINER:
--[[
local function HoldAltToSelectClassTrainer(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip, isFollowup)
  if (numGossip > 0 and (isFollowup or IsAltKeyDown())) then
    local title
    local class = UnitClass("player")
    for i=1,numGossip do
      title = gossip["title"..i]
      if (title == "Class Trainer") then
        return 3, i, HoldAltToSelectClassTrainer, false
      elseif (title == class) then
        return 3, i, nil, false
      end
    end
  end
end
Gossipmonger.SetFilter(HoldAltToSelectClassTrainer, "ALT for Class Trainer",
  'Hold ALT to automatically select "Class Trainer" and your class when talking to guards.', 2, true)
--]]


-- "COIN OF ANCESTRY COLLECTOR" (LUNAR FESTIVAL) DIALOG SKIPPING:
--[[
local function LunarFestivalVendor(mobID, NPCName, text, numAvailable, available, numActive, active, numGossip, gossip, isFollowup)
  if (numAvailable == 0 and numActive == 0 and numGossip == 2 and gossip.type1 == "vendor" and
      gossip.type2 == "gossip" and gossip.title2 == "Farewell.") then
    return 3, 1, nil, true
  end
end
Gossipmonger.SetFilter(LunarFestivalVendor, "Coin of Ancestry Collectors",
  "Skip past Coin of Ancestry vendors' dialog, forcing it to display in the gossip bubble. Blizzard apparently added the \"Farewell\" gossip option to the Lunar Festival vendors in Moonglade in order to get around their own auto-gossip-skipping code so players could see their special gossip text, but Gossipmonger's \"do both\" solution is arguably superior.",
  1, true)
--]]
